"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UUIDGenerator = void 0;
const random_1 = require("./random");
class UUIDGenerator {
    constructor(first, second) {
        this.random = new random_1.Random();
        this.staticData = '';
        this.version = '';
        this.variant = '';
        this.staticTargetLength = 12;
        if (first != null) {
            this.readFirstParam(first);
        }
        if (second) {
            this.staticData = this.convertStatic(second);
        }
        this.version = this.version || '4';
        this.staticData = this.staticData || this.generateStatic();
        this.variant = this.getVariant();
    }
    readFirstParam(first) {
        if (typeof first == 'number') {
            this.version = this.toString(first);
        }
        else if (typeof first == 'string') {
            this.staticData = this.convertStatic(first);
        }
    }
    getVariant() {
        return this.toString(this.random.next(16));
    }
    convertStatic(userStatic) {
        const data = this.getHashCode(userStatic);
        return data.length < this.staticTargetLength ? this.saltify(data) : data.slice(0, this.staticTargetLength);
    }
    generateStatic(length) {
        let staticData = '';
        const targetLength = length || this.staticTargetLength;
        for (let i = 0; i < targetLength; i++) {
            const rand = this.random.next(16);
            staticData += this.toString(rand);
        }
        return staticData;
    }
    /**
     * returns uuid
     */
    generate() {
        const date = this.getDateHEX().padStart(12, '0');
        const version = this.version.toString();
        const variant = this.variant.toString();
        const staticData = this.staticData;
        const random = this.generateStatic(6);
        return `${date.slice(0, 8)}-${date.slice(8, 12)}-${version}${random.slice(0, 3)}-${variant}${random.slice(3, 6)}-${staticData}`;
    }
    getDateHEX() {
        const date = new Date().getTime();
        return this.toString(date).replace('.', '');
    }
    getHashCode(str) {
        let hash = 0;
        for (let i = 0; i < str.length; i++) {
            hash = Math.imul(31, hash) + str.charCodeAt(i) | 0;
        }
        return this.toString((Math.abs(hash)));
    }
    saltify(hash) {
        const saltLength = this.staticTargetLength - hash.length;
        return hash + this.generateStatic(saltLength);
    }
    toString(number) {
        return number.toString(16);
    }
}
exports.UUIDGenerator = UUIDGenerator;
